@@ -1,6 +1,6 @@
 /*******************************************************************************
  *
- * Copyright (c) 2012-2017 Robert Krause (ruport@f00l.de)
+ * Copyright (c) 2012-2019 Robert Krause (ruport@f00l.de)
  *
  * This file is part of Pcapfix.
  *
@@ -20,52 +20,98 @@
 #include "pcapfix.h"
 #include "pcap.h"
 
-int nanoseconds = 0;			      /* pcap file uses nanoseconds (instead of microseconds) */
+/* pcap file global vars */
+int nanoseconds = 0;            /* pcap file uses nanoseconds (instead of microseconds) */
 
 /*
  * Function:  is_plausible
  * -----------------------
  * check if the pcap packet header could be a plausible one by satisfying those conditions:
- * - packet size >= 16 bytes AND <= 65535 bytes (included length AND original length) (conditions 1,2,3,4)
- * - included length <= original lenth (condition 5)
- * - packet timestamp is NOT older OR younger than the prior packets timestamp -+ one day (conditions 6,7)
- * - usec (microseconds) field <= 1000000 (conditions 8)
- * - usec (nanoseconds) field <= 1000000000 (conditions 9)
+ * - packet size >= 16 bytes AND <= MAX_SNAPLEN bytes (included length AND original length) (conditions 1,2,3,4,5)
+ * - included length <= original lenth (conditions 6,7,8,9)
+ * - packet timestamp is NOT older OR younger than the prior packets timestamp -+ one day (conditions 10,11,12)
+ * - usec (microseconds) field <= 1000000 (conditions 13)
+ * - usec (nanoseconds) field <= 1000000000 (conditions 14)
  *
- * hdr:       the filled packet header struct to check for plausibility
- * prior_ts:  the prior packets timestamp (seconds) to check for time relation (condition 6,7)
+ * global_hdr: the filled pcap header struct to check for snaplen
+ * hdr:        the filled packet header struct to check for plausibility
+ * prior_ts:   the prior packets timestamp (seconds) to check for time relation (condition 6,7)
  *
  * returns:  0   success
  *          -X   error (condition X failed)
  *
  */
-int is_plausible(struct packet_hdr_s hdr, unsigned int prior_ts) {
+int is_plausible(struct global_hdr_s global_hdr, struct packet_hdr_s hdr, unsigned int prior_ts) {
   /* check for minimum packet size
    * minimum packet size should be 16, but in some cases, e.g. local wlan capture, packet might
    * even be smaller --> decreased minimum size to 10 */
   if (conint(hdr.incl_len) < 10) return(-1);
   if (conint(hdr.orig_len) < 10) return(-2);
 
-  /* check max maximum packet size (0xffff) */
-  if (conint(hdr.incl_len) > 65535) return(-3);
-  if (conint(hdr.orig_len) > 65535) return(-4);
+  /* check packet limit with header field and max snaplen
+     in soft mode (and dlt 119) the packet detection accepts a wider range */
+  if (soft_mode || global_hdr.network == 119) {
+    /* in soft mode, global pcap snap length is ignored, instead MAX snaplen is used */
+
+    /* strange behavior for wlan: in some cases incl length seems to differ
+       not following the configured max snap length; this leads to dropping
+       valid wlan packets -> use soft mode for DLT 119 */
+    if (conint(hdr.incl_len) > PCAP_MAX_SNAPLEN) return(-3);
+  } else {
+    /* in hard mode (and not dlt 119), global pcap snap length is the limit in plausibility checks */
+    if (conint(hdr.incl_len) > conint(global_hdr.snaplen)) return(-4);
+  }
+  /* orig length should not be greater than pcap max snaplen */
+  if (conint(hdr.orig_len) > PCAP_MAX_SNAPLEN) return(-5);
+
+  /* if the packet original size is larger than the included length, then
+     the included length should be limited by the files snap length */
+  if (conint(hdr.orig_len) > conint(hdr.incl_len)) {
+    /* check for dlt 119 == wlan */
+    if (global_hdr.network == 119) {
+      /* strange behavior for wlan: in some cases incl length seems to differ
+         not following the configured max snap length; this leads to dropping
+         valid wlan packets -> ignore this check for DLT 119 */
+    } else {
+      if (conint(hdr.incl_len) != conint(global_hdr.snaplen)) return(-7);
+    }
+  }
 
   /* the included length CAN NOT be larger than the original length */
-  if (conint(hdr.incl_len) > conint(hdr.orig_len)) return(-5);
+  /* check for LINKTYPE_LINUX_SLL (linux cooked) */
+  if (global_hdr.network == 113) {
+    /* linux cooked headers are appended to packet length, but not to orig length
+       so we need to remove it from incl_len before checking */
+    if (conint(hdr.incl_len)-16 > conint(hdr.orig_len)) return(-8);
+  } else if (conint(hdr.incl_len) > conint(hdr.orig_len)) return(-9);
 
-  /* packet is not older than one day (related to prior packet) */
-  if ((prior_ts != 0) && (conint(hdr.ts_sec) > (prior_ts+86400))) return(-6);
-
-  /* packet is not younger than one day (related to prior packet) */
-  if ((prior_ts >= 86400) && (conint(hdr.ts_sec) < (prior_ts-86400))) return(-7);
-
-  /* check for nano/microseconds */
-  if (nanoseconds == 0) {
-    /* usec (microseconds) must <= 1000000 */
-    if (conint(hdr.ts_usec) > 1000000) return(-8);
+  /* check packet times (older) */
+  if (soft_mode) {
+    /* in soft mode, there is no limit for older packets */
   } else {
-    /* usec (nanoseconds) must be <= 1000000000 */
-    if (conint(hdr.ts_usec) > 1000000000) return(-9);
+    /* in hard mode, packet must not be older than one day (related to prior packet) */
+    if ((prior_ts != 0) && (conint(hdr.ts_sec) > (prior_ts+86400))) return(-10);
+  }
+
+  /* check packet times (younger) */
+  if (soft_mode) {
+    /* in soft mode, packet must not be younger than one day (related to prior packet) */
+    if ((prior_ts >= 86400) && (conint(hdr.ts_sec) < (prior_ts-86400))) return(-11);
+  } else {
+    /* in hard mode, packets must not be younger than prior packet */
+    if ((prior_ts != 0) && (conint(hdr.ts_sec) < prior_ts)) return(-12);
+  }
+
+  /* check for nano/microseconds (hard mode only) */
+  if (!soft_mode) {
+    /* in hard mode */
+    if (nanoseconds == 0) {
+      /* usec (microseconds) must <= 1000000 */
+      if (conint(hdr.ts_usec) > 1000000) return(-13);
+    } else {
+      /* usec (nanoseconds) must be <= 1000000000 */
+      if (conint(hdr.ts_usec) > 1000000000) return(-14);
+    }
   }
 
   /* all conditions fullfilled ==> everything fine! */
@@ -77,23 +123,24 @@ int is_plausible(struct packet_hdr_s hdr, unsigned int prior_ts) {
  * -----------------------
  * this function takes a buffer and brute forces some possible ascii-corrupted bytes versus plausibility checks
  *
- * buffer:   the buffer that might contain the possible pcap packet header
- * size:     the size of the buffer (double pcap packet header size is a good choice)
- * priot_ts: the prior packets timestamp (to check for plausibility)
- * hdr:      the pointer to the packet header buffer (we use this to return the repaired header)
+ * buffer:     the buffer that might contain the possible pcap packet header
+ * size:       the size of the buffer (double pcap packet header size is a good choice)
+ * priot_ts:   the prior packets timestamp (to check for plausibility)
+ * global_hdr: the pointer to the pcap header
+ * hdr:        the pointer to the packet header buffer (we use this to return the repaired header)
  *
  * returns: >=0   success (return value contains number of ascii corrupted bytes in hdr (we need this data to align the beginning of the packet body later)
  *           -1   error (no valid pcap header found inside buffer)
  *
  */
-int check_header(char *buffer, unsigned int size, unsigned int prior_ts, struct packet_hdr_s *hdr) {
+int check_header(char *buffer, unsigned int size, unsigned int prior_ts, struct global_hdr_s *global_hdr, struct packet_hdr_s *hdr) {
   unsigned int i; /* loop variable - first byte in buffer that could be beginning of packet */
   int res;        /* return value */
   char *tmp;      /* the temporary buffer that will be used for recursion */
 
   /* does the buffer already contain a valid packet header (without any correction) ?? */
   memcpy(hdr, buffer, sizeof(struct packet_hdr_s));
-  res = is_plausible(*hdr, prior_ts);
+  res = is_plausible(*global_hdr, *hdr, prior_ts);
   if (res == 0) return(0);
 
   if (verbose >= 2) printf("[-] Header plausibility check failed with error code %d\n", res);
@@ -115,7 +162,7 @@ int check_header(char *buffer, unsigned int size, unsigned int prior_ts, struct
       memcpy(tmp+i, buffer+i+1, size-i-1);
 
       /* and invoke the header again without this 0x0D byte */
-      res = check_header(tmp, size-1, prior_ts, hdr);
+      res = check_header(tmp, size-1, prior_ts, global_hdr, hdr);
 
       /* free recursion buffer */
       free(tmp);
@@ -145,31 +192,16 @@ int check_header(char *buffer, unsigned int size, unsigned int prior_ts, struct
  *
  */
 int fix_pcap(FILE *pcap, FILE *pcap_fix) {
-  struct global_hdr_s global_hdr;		      /* global header data */
-  struct packet_hdr_s packet_hdr;		      /* packet header data */
-  struct packet_hdr_s next_packet_hdr;		  /* next packet header data to look forward */
+  struct global_hdr_s global_hdr;		/* global header data */
 
-  char hdrbuffer[sizeof(packet_hdr)*2];		  /* the buffer that will be used to find a proper packet */
-  char buffer[65535];				          /* the packet body */
-
-  // we use a buffer to cache 1mb of writing... this way writing is faster and
-  // we can read and write the file at the same time
+  /* we use a buffer to cache 1mb of writing... this way writing is faster and
+     we can read and write the file at the same time */
   char *writebuffer;
   uint64_t writepos = 0;
 
-  uint64_t pos = 0;			                  /* position of current packet header */
-  uint64_t nextpos = 0;			              /* possible position of next packets header */
-  uint64_t bytes;				              /* read/written bytes counter (unused yet) */
-  uint64_t filesize;                          /* filesize of input file in bytes */
-  unsigned int count;				          /* packet counter */
-  unsigned int step = 1;                      /* step counter for progress bar */
-  unsigned int last_correct_ts_sec = 0;		  /* timestamp of the last proper packet found (seconds) */
-  unsigned int last_correct_ts_usec = 0;	  /* timestamp of the last proper packet found (microseconds or nanoseconds) */
-  unsigned short hdr_integ;			          /* integrity counter of global header */
-
-  int ascii = 0;				              /* ascii counter for possible ascii-corrupted packets */
-  int corrupted = 0;				          /* corrupted packet counter for final output */
-  int res;					                  /* the result of the header check == the offset of body shifting */
+  uint64_t bytes;				/* read/written bytes counter */
+  uint64_t filesize;				/* filesize of input file in bytes */
+  unsigned short hdr_integ;			/* integrity counter of global header */
 
   /* init write buffer */
   writebuffer = malloc(1024000);
@@ -257,12 +289,12 @@ int fix_pcap(FILE *pcap, FILE *pcap_fix) {
   }
 
   /* check for max packet length */
-  if ((conint(global_hdr.snaplen) > 0) && (conint(global_hdr.snaplen) <= 65535)) {	/* typically 65535 (no support for huge packets yet) */
+  if ((conint(global_hdr.snaplen) > 0) && (conint(global_hdr.snaplen) <= PCAP_MAX_SNAPLEN)) {	/* typically 262144 nowadays */
     if (verbose) printf("[+] Max packet length: %u\n", conint(global_hdr.snaplen));
   } else {
     hdr_integ++;
     if (verbose) printf("[-] Max packet length: %u\n", conint(global_hdr.snaplen));
-    global_hdr.snaplen = conint(65535);
+    global_hdr.snaplen = conint(PCAP_MAX_SNAPLEN);
   }
 
   /* check for data link type (http://www.tcpdump.org/linktypes.html) */
@@ -293,7 +325,7 @@ int fix_pcap(FILE *pcap, FILE *pcap_fix) {
     fseeko(pcap, 0, SEEK_SET);
 
     /* set important header values to defaults */
-    global_hdr.snaplen = conint(65535);
+    global_hdr.snaplen = conint(PCAP_MAX_SNAPLEN);
 
     if (data_link_type != -1) {
       global_hdr.network = conint(data_link_type);
@@ -310,6 +342,52 @@ int fix_pcap(FILE *pcap, FILE *pcap_fix) {
 
   /* END OF GLOBAL HEADER CHECK */
 
+  /* start checking packets now */
+  return(fix_pcap_packets(pcap, pcap_fix, filesize, global_hdr, hdr_integ, writebuffer, writepos));
+}
+
+/*
+ * Function:  fix_pcap_packets
+ * ---------------------------
+ * tries to fix pcap packets inside a pcap file
+ *
+ * pcap:        file pointer to input file
+ * pcap_fix:    file pointer to output file
+ * filesize:    input file size in bytes
+ * global_hdr:  global header struct
+ * hdr_integ:   global header integrity counter
+ * writebuffer: buffer to write pcap packets into
+ * writepos:    position in buffer to write next packet to
+ *
+ * returns: >0   success (number of corruptions fixed)
+ *           0   success (nothing to fix)
+ *          -1   error (not a pcap file)
+ *          -2   error (unable to repair)
+ *          -3   error (EOF reached while reading input file)
+ *
+ */
+int fix_pcap_packets(FILE *pcap, FILE *pcap_fix, uint64_t filesize, struct global_hdr_s global_hdr, unsigned short hdr_integ, char *writebuffer, uint64_t writepos) {
+  struct packet_hdr_s packet_hdr;		/* packet header data */
+  struct packet_hdr_s next_packet_hdr;		/* next packet header data to look forward */
+  char hdrbuffer[sizeof(packet_hdr)*2];		/* the buffer that will be used to find a proper packet */
+
+  char buffer[PCAP_MAX_SNAPLEN];		/* the packet body */
+  uint64_t bytes;				/* read/written bytes counter */
+
+  unsigned int count;				/* packet counter */
+  unsigned int step = 1;			/* step counter for progress bar */
+
+  uint64_t pos = 0;				/* position of current packet header */
+  uint64_t nextpos = 0;			        /* possible position of next packets header */
+
+  unsigned int last_correct_ts_sec = 0;		/* timestamp of the last proper packet found (seconds) */
+  unsigned int last_correct_ts_usec = 0;	/* timestamp of the last proper packet found (microseconds or nanoseconds) */
+
+  int ascii = 0;				/* ascii counter for possible ascii-corrupted packets */
+  int corrupted = 0;				/* corrupted packet counter for final output */
+
+  int res;					/* the result of the header check == the offset of body shifting */
+
   /* BEGIN PACKET CHECK */
 
   printf("[*] Analyzing packets...\n");
@@ -355,22 +433,22 @@ int fix_pcap(FILE *pcap, FILE *pcap_fix) {
     if (bytes != 1) return -3;
 
     /* check if the packet header looks proper */
-    res = check_header(hdrbuffer, sizeof(hdrbuffer), last_correct_ts_sec, &packet_hdr);
+    res = check_header(hdrbuffer, sizeof(hdrbuffer), last_correct_ts_sec, &global_hdr, &packet_hdr);
     if (res != -1) {
 
       /* realign packet body (based on possible-ascii corrupted pcap header) */
       pos += res;
-      fseeko(pcap, pos+16, SEEK_SET);
+      fseeko(pcap, pos+sizeof(packet_hdr), SEEK_SET);
 
       /* try to read the packet body AND check if there are still at least 16 bytes left for the next pcap packet header */
-      if ((fread(&buffer, conint(packet_hdr.incl_len), 1, pcap) == 0) || ((filesize-(pos+16+conint(packet_hdr.incl_len)) > 0) && (filesize-(pos+16+conint(packet_hdr.incl_len)) < 16))) {
+      if ((fread(&buffer, conint(packet_hdr.incl_len), 1, pcap) == 0) || ((filesize-(pos+sizeof(packet_hdr)+conint(packet_hdr.incl_len)) > 0) && (filesize-(pos+sizeof(packet_hdr)+conint(packet_hdr.incl_len)) < sizeof(packet_hdr)))) {
         /* fread returned an error (EOL while read the body) or the file is not large enough for the next pcap packet header (16bytes) */
         /* thou the last packet has been cut of */
 
         if (verbose >= 1) printf("[-] LAST PACKET MISMATCH (%u | %u | %u | %u)\n", conint(packet_hdr.ts_sec), conint(packet_hdr.ts_usec), conint(packet_hdr.incl_len), conint(packet_hdr.orig_len));
 
         /* correct the packets included length field to match the end of file */
-        packet_hdr.incl_len = conint(filesize-pos-16);
+        packet_hdr.incl_len = conint(filesize-pos-sizeof(packet_hdr));
 
         /* the original length must not be smaller than the included length */
         if (conint(packet_hdr.incl_len) > conint(packet_hdr.orig_len)) packet_hdr.orig_len = packet_hdr.incl_len;
@@ -387,29 +465,32 @@ int fix_pcap(FILE *pcap, FILE *pcap_fix) {
       bytes = fread(hdrbuffer, sizeof(hdrbuffer), 1, pcap);
 
       /* check if next packets header looks proper */
-      if (check_header(hdrbuffer, sizeof(hdrbuffer), conint(packet_hdr.ts_sec), &next_packet_hdr) == -1) {
+      if (check_header(hdrbuffer, sizeof(hdrbuffer), conint(packet_hdr.ts_sec), &global_hdr, &next_packet_hdr) == -1) {
 
         /* the next packets header is corrupted thou we are going to scan through the prior packets body to look for an overlapped packet header
          * also look inside the next packets header + 16bytes of packet body, because we need to know HERE
          * do not leave the loop if the first packet has not been found yet AND deep scan mode is activated */
-        for (nextpos=pos+16+1; (nextpos < pos+16+conint(packet_hdr.incl_len)+32) || (count == 1 && deep_scan == 1); nextpos++) {
+        for (nextpos=pos+sizeof(packet_hdr)+1; (nextpos < pos+sizeof(packet_hdr)+conint(packet_hdr.incl_len)+2*sizeof(packet_hdr)) || (count == 1 && deep_scan == 1); nextpos++) {
 
           /* read the possible next packets header */
           fseeko(pcap, nextpos, SEEK_SET);
           bytes = fread(hdrbuffer, sizeof(hdrbuffer), 1, pcap);
 
+          /* check read success */
+          if (bytes == 0) break;
+
           /* heavy verbose output :-) */
           if (verbose >= 2) printf("[*] Trying Packet #%u at position %" PRIu64 " (%u | %u | %u | %u).\n", (count+1), nextpos, conint(next_packet_hdr.ts_sec), conint(next_packet_hdr.ts_usec), conint(next_packet_hdr.incl_len), conint(next_packet_hdr.orig_len));
 
           /* check the header for plausibility */
-          res = check_header(hdrbuffer, sizeof(hdrbuffer), last_correct_ts_sec, &next_packet_hdr);
+          res = check_header(hdrbuffer, sizeof(hdrbuffer), last_correct_ts_sec, &global_hdr, &next_packet_hdr);
           if (res != -1) {
 
             /* we found a proper header inside the packets body! */
             if (verbose >= 1) printf("[-] FOUND OVERLAPPING data of Packet #%u at position %" PRIu64 " (%u | %u | %u | %u).\n", (count+1), nextpos, conint(next_packet_hdr.ts_sec), conint(next_packet_hdr.ts_usec), conint(next_packet_hdr.incl_len), conint(next_packet_hdr.orig_len));
 
             /* correct the prior packets length information fields to align the overlapped packet */
-            packet_hdr.incl_len = conint(nextpos-(pos+16)+res);	/* also include ascii corruption offset (res) */
+            packet_hdr.incl_len = conint(nextpos-(pos+sizeof(packet_hdr))+res);	/* also include ascii corruption offset (res) */
             packet_hdr.orig_len = packet_hdr.incl_len;
 
             /* time correction for the FIRST packet only */
@@ -442,7 +523,7 @@ int fix_pcap(FILE *pcap, FILE *pcap_fix) {
       }
 
       /* reset file fointer to next packet */
-      fseeko(pcap, pos+16+conint(packet_hdr.incl_len), SEEK_SET);
+      fseeko(pcap, pos+sizeof(packet_hdr)+conint(packet_hdr.incl_len), SEEK_SET);
 
       /* we found a correct packet (and aligned it maybe) */
       if (verbose >= 1) printf("[+] Packet #%u at position %" PRIu64 " (%u | %u | %u | %u).\n", count, pos, conint(packet_hdr.ts_sec), conint(packet_hdr.ts_usec), conint(packet_hdr.incl_len), conint(packet_hdr.orig_len));
@@ -474,7 +555,7 @@ int fix_pcap(FILE *pcap, FILE *pcap_fix) {
 
       /* scan from the current position to the maximum packet size and look for a next proper packet header to align the corrupted packet
        * also do not leave the loop if the first packet has not been found yet AND deep scan mode is activated */
-      for (nextpos=pos+16+1; (nextpos <= pos+16+65535) || (count == 1 && deep_scan == 1); nextpos++) {
+      for (nextpos=pos+sizeof(packet_hdr)+1; (nextpos <= pos+sizeof(packet_hdr)+PCAP_MAX_SNAPLEN) || (count == 1 && deep_scan == 1); nextpos++) {
 
         /* read the possible next packets header */
         fseeko(pcap, nextpos, SEEK_SET);
@@ -491,8 +572,19 @@ int fix_pcap(FILE *pcap, FILE *pcap_fix) {
 
           printf("[*] End of file reached. Aligning last packet.\n");
 
+          /* check last packet for plausiblilty */
+          res = check_header(hdrbuffer, sizeof(hdrbuffer), last_correct_ts_sec, &global_hdr, &packet_hdr);
+          if (res != 0) {
+            printf("[-] Cannot align last packet, because it is broken.\n");
+            corrupted++;
+            count--;
+            break;
+          }
+
+          /* check passed */
+
           /* align the last packet to match EOF */
-          packet_hdr.incl_len = conint(filesize-(pos+16));
+          packet_hdr.incl_len = conint(filesize-(pos+sizeof(packet_hdr)));
           packet_hdr.orig_len = packet_hdr.incl_len;
 
           /* if the is the first packet, we need to set timestamps to zero */
@@ -505,7 +597,7 @@ int fix_pcap(FILE *pcap, FILE *pcap_fix) {
           }
 
           /* read the packets body (size based on the just found next packets position) */
-          fseeko(pcap, pos+16, SEEK_SET);
+          fseeko(pcap, pos+sizeof(packet_hdr), SEEK_SET);
           bytes = fread(&buffer, conint(packet_hdr.incl_len), 1, pcap);
 
           // check if there is enough space in buffer
@@ -539,11 +631,11 @@ int fix_pcap(FILE *pcap, FILE *pcap_fix) {
         if (verbose >= 2) printf("[*] Trying Packet #%u at position %" PRIu64 " (%u | %u | %u | %u).\n", (count+1), nextpos, conint(next_packet_hdr.ts_sec), conint(next_packet_hdr.ts_usec), conint(next_packet_hdr.incl_len), conint(next_packet_hdr.orig_len));
 
         /* check if next packets header looks proper */
-        res = check_header(hdrbuffer, sizeof(hdrbuffer), last_correct_ts_sec, &next_packet_hdr);
+        res = check_header(hdrbuffer, sizeof(hdrbuffer), last_correct_ts_sec, &global_hdr, &next_packet_hdr);
         if (res != -1) {
 
-          /* if we found a packet that is below the top 65535 bytes (deep scan) we cut it off and take the second packet as first one */
-          if ((nextpos-(pos+16) > 65535) && (count == 1) && (deep_scan == 1)) {
+          /* if we found a packet that is below the top MAX_SNAPLEN bytes (deep scan) we cut it off and take the second packet as first one */
+          if ((nextpos-(pos+sizeof(packet_hdr)) > PCAP_MAX_SNAPLEN) && (count == 1) && (deep_scan == 1)) {
 
             if (verbose >= 1) printf("[+] (DEEP SCAN) FOUND FIRST Packet #%u at position %" PRIu64 " (%u | %u | %u | %u).\n", count, nextpos, conint(next_packet_hdr.ts_sec), conint(next_packet_hdr.ts_usec), conint(next_packet_hdr.incl_len), conint(next_packet_hdr.orig_len));
 
@@ -559,7 +651,7 @@ int fix_pcap(FILE *pcap, FILE *pcap_fix) {
             if (verbose >= 1) printf("[+] FOUND NEXT Packet #%u at position %" PRIu64 " (%u | %u | %u | %u).\n", (count+1), nextpos, conint(next_packet_hdr.ts_sec), conint(next_packet_hdr.ts_usec), conint(next_packet_hdr.incl_len), conint(next_packet_hdr.orig_len));
 
             /* correct the corrupted pcap packet header to match the just found next packet header */
-            packet_hdr.incl_len = conint(nextpos-(pos+16));
+            packet_hdr.incl_len = conint(nextpos-(pos+sizeof(packet_hdr)));
             packet_hdr.orig_len = packet_hdr.incl_len;
 
             if (count == 1) { /* time correction for the FIRST packet */
@@ -582,11 +674,11 @@ int fix_pcap(FILE *pcap, FILE *pcap_fix) {
             }
 
             /* read the packets body (size based on the just found next packets position) */
-            fseeko(pcap, pos+16, SEEK_SET);
+            fseeko(pcap, pos+sizeof(packet_hdr), SEEK_SET);
             bytes = fread(&buffer, conint(packet_hdr.incl_len), 1, pcap);
 
             /* final check resulting packet for plausibility */
-            res = is_plausible(packet_hdr, last_correct_ts_sec);
+            res = is_plausible(global_hdr, packet_hdr, last_correct_ts_sec);
             if (res == 0) {
 
               // check if there is enough space in buffer
@@ -629,7 +721,7 @@ int fix_pcap(FILE *pcap, FILE *pcap_fix) {
       if (corrupted == -1) break;
 
       /* did the counter exceed the maximum packet size? */
-      if ((count == 1 && deep_scan == 0) && (nextpos > pos+16+65535)) {
+      if ((count == 1 && deep_scan == 0) && (nextpos > pos+sizeof(packet_hdr)+PCAP_MAX_SNAPLEN)) {
 
         /* PACKET COULD NOT BE REPAIRED! */
 
@@ -638,7 +730,7 @@ int fix_pcap(FILE *pcap, FILE *pcap_fix) {
       }
 
       /* maximum search range reached? -> skip packet and keep on searching */
-      if (deep_scan == 0 && (nextpos > pos+16+65535)) {
+      if (deep_scan == 0 && (nextpos > pos+sizeof(packet_hdr)+PCAP_MAX_SNAPLEN)) {
         if (verbose >= 1) printf("[-] No next packet found within max packet range --> SKIPPING!\n");
 
         /* reset counter because no packet found */
@@ -690,6 +782,11 @@ int fix_pcap(FILE *pcap, FILE *pcap_fix) {
     }
 
   } else {
+    /* check if we found at least one packet */
+    if (count == 1) {
+      /* even the first packet was corrupted and no other packet could be found */
+      return(-1);
+    }
 
     /* file has been successfully repaired (corruption fixed) */
 
