@@ -1,6 +1,6 @@
 /*******************************************************************************
  *
- * Copyright (c) 2012-2017 Robert Krause (ruport@f00l.de)
+ * Copyright (c) 2012-2019 Robert Krause (ruport@f00l.de)
  *
  * This file is part of Pcapfix.
  *
@@ -15,7 +15,7 @@
  * You should have received a copy of the GNU General Public License along with
  * Pcapfix. If not, see http://www.gnu.org/licenses/.
  *
- * Last Modified: 25.12.2017
+ * Last Modified: 22.01.2019
  *
  *******************************************************************************
  *
@@ -42,9 +42,10 @@
 
 #include "pcapfix.h"
 #include "pcap.h"
+#include "pcap_kuznet.h"
 #include "pcapng.h"
 
-#define VERSION "1.1.1"			    /* pcapfix version */
+#define VERSION "1.1.4"			    /* pcapfix version */
 
 #define BTSNOOP_MAGIC 0x6E737462    /* btsnoop file magic (first 4 bytes) */
 #define SNOOP_MAGIC 0x6f6f6e73	    /* snoop file magic (first 4 bytes) */
@@ -53,11 +54,13 @@
 #define ETHERPEEK_MAGIC 0x7265767f  /* EtherPeek/AiroPeek/OmniPeek file magic */
 
 /* configuration variables */
-int deep_scan = 0;		    /* deep scan option (default: no deep scan) */
-int verbose = 0;			/* verbose output option (default: dont be verbose) */
-int swapped = 0;			/* pcap file is swapped (big endian) */
-int data_link_type = -1;    /* data link type (default: LINKTYPE_ETHERNET) */
-int pcapng = 0;             /* file format to assume */
+int deep_scan = 0;		/* deep scan option (default: no deep scan) */
+int soft_mode = 0;		/* soft mode option (default: no soft mode) */
+int keep_outfile = 0;		/* keep output file even if nothing needed fixing (default: don't) */
+int verbose = 0;		/* verbose output option (default: dont be verbose) */
+int swapped = 0;		/* pcap file is swapped (big endian) */
+int data_link_type = -1;	/* data link type (default: LINKTYPE_ETHERNET) */
+int pcapng = 0;			/* file format to assume */
 
 /* header placeholder */
 unsigned int header_magic;
@@ -75,8 +78,10 @@ void usage(char *progname) {
   printf("Usage: %s [OPTIONS] filename\n", progname);
   printf("OPTIONS:");
   printf(  "\t-d        , --deep-scan          \tDeep scan (pcap only)\n");
+  printf("\t\t-s        , --soft-mode          \tSoft mode (packet detection)\n");
   printf("\t\t-n        , --pcapng             \tforce pcapng format\n");
   printf("\t\t-o <file> , --outfile <file>     \tset output file name\n");
+  printf("\t\t-k        , --keep-outfile       \tdon't delete the output file if nothing needed to be fixed\n");
   printf("\t\t-t <nr>   , --data-link-type <nr>\tData link type\n");
   printf("\t\t-v        , --verbose            \tVerbose output\n");
   printf("\n");
@@ -176,25 +181,33 @@ int main(int argc, char *argv[]) {
 
   /* init getopt_long options struct */
   struct option long_options[] = {
-    {"deep-scan", no_argument, 0, 'd'},				        /* --deep-scan == -d */
-    {"pcapng", no_argument, 0, 'n'},				        /* --pcapng == -n */
-    {"outfile", required_argument, 0, 'o'},		    /* --outfile == -o */
-    {"data-link-type", required_argument, 0, 't'},		    /* --data-link-type == -t */
-    {"verbose", no_argument, 0, 'v'},				        /* --verbose == -v */
+    {"deep-scan", no_argument, 0, 'd'},            /* --deep-scan == -d */
+    {"soft-mode", no_argument, 0, 's'},            /* --soft-mode == -s */
+    {"pcapng", no_argument, 0, 'n'},               /* --pcapng == -n */
+    {"outfile", required_argument, 0, 'o'},        /* --outfile == -o */
+    {"keep-outfile", no_argument, 0, 'k'},         /* --keep-outfile == -k */
+    {"data-link-type", required_argument, 0, 't'}, /* --data-link-type == -t */
+    {"verbose", no_argument, 0, 'v'},              /* --verbose == -v */
     {0, 0, 0, 0}
   };
 
   /* print out pcapfix header information */
-  printf("pcapfix %s (c) 2012-2014 Robert Krause\n\n", VERSION);
+  printf("pcapfix %s (c) 2012-2019 Robert Krause\n\n", VERSION);
 
   /* scan for options and arguments */
-  while ((c = getopt_long(argc, argv, ":t:o:v::d::n::", long_options, &option_index)) != -1) {
+  while ((c = getopt_long(argc, argv, ":t:ko:v::d::s::n::", long_options, &option_index)) != -1) {
     switch (c) {
       case 0:	/* getopt_long options evaluation */
         break;
       case 'd':	/* deep scan */
         deep_scan++;
         break;
+      case 's':	/* soft mode */
+        soft_mode++;
+        break;
+      case 'k': /* keep outfile even if nothing needed fixing */
+        keep_outfile++;
+        break;
       case 'n':	/* pcapng format */
         pcapng++;
         break;
@@ -353,19 +366,13 @@ int main(int argc, char *argv[]) {
 
       return(-6);
 
+    /* extended pcap format (KUZNETZOV) */
     case PCAP_EXT_MAGIC:
     case PCAP_EXT_MAGIC_SWAPPED:
-      printf("[-] This is an extended tcpdump file, which is not supported, *YET*!\n");
-      printf("[*] If you need really need this filetype to be supported write a brief email to ruport@f00l.de\n\n");
+      printf("[+] This is an extended tcpdump file.\n");
+      res = fix_pcap_kuznetzov(pcap, pcap_fix);
 
-      /* close input and output files */
-      fclose(pcap);
-      fclose(pcap_fix);
-
-      /* delete output file due to no changes failure */
-      if (strcmp(filename, filename_fix) != 0) remove(filename_fix);
-
-      return(-6);
+      break;
 
     /* PCAPNG format */
     case PCAPNG_MAGIC:
@@ -409,7 +416,9 @@ int main(int argc, char *argv[]) {
       fclose(pcap_fix);
 
       /* delete output file due to no changes failure */
-      if (strcmp(filename, filename_fix) != 0) remove(filename_fix);
+      if ((strcmp(filename, filename_fix) != 0) && (0 == keep_outfile)) {
+        remove(filename_fix);
+      }
 
       return(0);
 
@@ -424,11 +433,9 @@ int main(int argc, char *argv[]) {
       /* delete output file due to failure */
       if (strcmp(filename, filename_fix) != 0) remove(filename_fix);
 
-      /* deep scan dependent output */
-      if (pcapng == 0) {
-        if (deep_scan == 0) printf("If you are SURE that there are pcap packets inside, try with deep scan option (-d) to find them!\n\n");
-        else printf("There is REALLY no pcap packet inside this file!!!\n\n");
-      }
+      /* deep scan / soft mode dependent output */
+      if (deep_scan == 0 || soft_mode == 0) printf("If you are SURE that there are pcap packets inside, try with deep scan (-d) (pcap only!) and/or soft mode (-s) to find them!\n\n");
+      else printf("There is REALLY no pcap packet inside this file!!!\n\n");
 
       return(res-10);
 
@@ -473,13 +480,13 @@ int main(int argc, char *argv[]) {
     /* Successful repaired! (res > 0) */
 
     fclose(pcap);
-    int finalpos = ftello(pcap_fix);
+    off_t finalpos = ftello(pcap_fix);
     fclose(pcap_fix);
     int success = truncate(filename_fix, finalpos);
     if (success != 0) printf("[-] Truncating result file failed!");
 
     printf("[+] SUCCESS: %d Corruption(s) fixed!\n\n", res);
-    return(1);
+    return(0);
 
   } else {
     /* Unknown Error (res < 0); this should NEVER happen! */
