@@ -1,6 +1,6 @@
 /*******************************************************************************
  *
- * Copyright (c) 2012-2017 Robert Krause (ruport@f00l.de)
+ * Copyright (c) 2012-2019 Robert Krause (ruport@f00l.de)
  *
  * This file is part of Pcapfix.
  *
@@ -577,6 +577,9 @@ int fix_pcapng(FILE *pcap, FILE *pcap_fix) {
         if (bytes != 1) return -3;
         left -= sizeof(spb);
 
+        /* output data */
+        if (verbose >= 1) printf("[*] SPB original packet length: %u bytes\n", spb.len);
+
         /* copy simple packet block into repaired file */
         memcpy(new_block+block_pos, &spb, sizeof(spb));
         block_pos += sizeof(spb);
@@ -1451,9 +1454,10 @@ int find_valid_block(FILE *pcap, uint64_t filesize) {
   uint64_t i;
   unsigned int bytes;
   unsigned int check;                       /* variable to check end of blocks sizes */
-  struct block_header bh;
+  struct block_header bh;                   /* block header */
   struct packet_block pb;                   /* Packet Block */
   struct name_resolution_block nrb;         /* Name Resolution Block */
+  struct simple_packet_block spb;           /* Simple Packet Block */
 
   /* bytewise processing of input file */
   for (i=ftello(pcap)-4; i<filesize; i++) {
@@ -1483,10 +1487,16 @@ int find_valid_block(FILE *pcap, uint64_t filesize) {
       }
 
       /* Simple Packet Block Checks:
-       * - max size <= 65535 */
+       * - max size <= MAX_SNAPLEN */
       if (bh.block_type == TYPE_SPB) {
         /* max size check */
-        if (bh.total_length > 65536) continue;
+        if (bh.total_length > PCAPNG_MAX_SNAPLEN) continue;
+
+        bytes = fread(&spb, sizeof(spb), 1, pcap);
+        if (bytes != 1) return(-1);
+
+        /* check original packet lengths <= MAX_SNAPLEN */
+        if (spb.len > PCAPNG_MAX_SNAPLEN) continue;
       }
 
       /* Name Resolution Block Checks:
@@ -1670,7 +1680,7 @@ int write_idb(FILE *pcap_fix, char* writebuffer, uint64_t* writepos) {
   idb.reserved = 0;
 
   /* we set snaplen to maximum */
-  idb.snaplen = 65535;
+  idb.snaplen = PCAPNG_MAX_SNAPLEN;
 
   /* increase total size by interface desciption block (header) */
   size += sizeof(struct interface_description_block);
